
# !/bin/ruby
# -*- encoding : utf-8 -*-
require_relative 'spec_helper'
require_relative '../lib/pacman/controller'

# Pacman module
module Pacman
  describe LevelBuilder do
    describe 'DSL' do
      let(:dsl1) do
        <<-EOS
      resize 10, 8
      EOS
      end
      it 'resizes maze' do
        level = described_class.build(dsl1)
        expect(level.width).to eq 10
        expect(level.height).to eq 8
      end

      let(:dsl2) do
        <<-EOS
      resize 4, 3
      wall
      wall
      skip 3
      wall
      EOS
      end

      it '#add_wall and #skip' do
        level = described_class.build(dsl2)
        expect(level[0][0].instance_of?(Wall)).to be true
        expect(level[0][1].instance_of?(Wall)).to be true
        expect(level[0][2]).to be nil
        expect(level[1][0]).to be nil
        expect(level[1][1].instance_of?(Wall)).to be true
        expect(level[1][2]).to be nil
      end

      let(:dsl3) do
        <<-EOS
      resize 4, 2
      goto 2, 0
      wall
      goto 3, 1
      wall
      EOS
      end

      it '#goto and place to the last position' do
        level = described_class.build(dsl3)
        expect(level[0][2].instance_of?(Wall)).to be true
        expect(level[1][2]).to be nil
        expect(level[0][3]).to be nil
        expect(level[1][3].instance_of?(Wall)).to be true
      end

      let(:dsl4) do
        <<-EOS
      resize 4, 2
      pellet
      pellet
      skip 1
      power_pellet
      pellet
      wall
      EOS
      end

      it '#pellet and #power_pellet' do
        level = described_class.build(dsl4)
        expect(level[0][0].instance_of?(Pellet)).to be true
        expect(level[0][1].instance_of?(Pellet)).to be true
        expect(level[0][2]).to be nil
        expect(level[0][3].instance_of?(PowerPellet)).to be true
        expect(level[1][0].instance_of?(Pellet)).to be true
      end

      let(:dsl5) do
        <<-EOS
      resize 4, 2
      goto 2, 1
      player :left
      EOS
      end

      it '#player set position' do
        level = described_class.build(dsl5)
        expect(level.player.instance_of?(Player)).to be true
        expect(level.player.direction).to be :left
      end

      let(:dsl6) do
        <<-EOS
      resize 7, 4
      goto 1, 1
      cage
      wall
      EOS
      end

      it '#cage at specified position' do
        level = described_class.build(dsl6)
        expect(level.get(1, 1).instance_of?(Cage)).to be true
        expect(level.get(2, 1).instance_of?(Wall)).to be true
      end
    end
  end
end
