
# !/bin/ruby
# -*- encoding : utf-8 -*-
require_relative 'spec_helper'
require_relative '../lib/pacman/controller'

# Pacman module
module Pacman
  describe Level do
    subject(:level) { Level.new }

    describe 'iteration' do
      it 'method #resize' do
        expect(level.methods.include?(:resize)).to be true
        level.resize(9, 8)
        expect(level.width).to eq 9
        expect(level.height).to eq 8
      end

      it 'has method #each' do
        expect(level.methods.include?(:each)).to be true
      end

      it '#each without block' do
        expect(level.each.instance_of?(Enumerator)).to be true
        expect(level.each.size).to eq 0
        level.resize(10, 5)
        expect(level.each.size).to eq 5
        expect(level[0].size).to eq 10
      end

      it '#each with a block' do
        level.resize(6, 4)
        count = 0
        level.each do |row|
          row.each do |_col|
            count += 2
          end
        end
        expect(count).to eq 6 * 4 * 2
      end

      it 'has method #each_with_position' do
        expect(level.methods.include?(:each_with_position)).to be true
        level.resize(3, 4)
        positions = []

        count = 0
        level.each_with_position do |x, y, _obj|
          count += 1
          fail('should not repeat elements position') if positions.include?("#{x},#{y}")
          positions << "#{x},#{y}"
        end
        expect(count).to be 3 * 4
      end
    end

    describe 'accessing:' do
      let(:data) { [11, 12] }
      it 'access row via [] and []=' do
        level.resize(2, 2)
        level[1] = data
        expect(level[0]).to eq [nil, nil]
        expect(level[1]).to eq data
      end

    end
  end
end
